/*
 * Spirit.java
 *
 * Created on February 8, 2004, 11:08 PM
 */

package MySimulation;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.lang.*;
import java.util.*;
import javax.swing.JPanel;

/**
 *
 * @author  Kevin Chiou
 */

/** Sprite class allows to create moving objects on the screen. */

class Sprite extends Observable {
    //constants that can be used outside the class
    final static int NORTH = 180;
    final static int SOUTH = 0;
    final static int WEST = 270;
    final static int EAST = 90;
    final static int MAXSPEED = 100;
    
    int drawextra = 0;

    protected final static int DELTA = 4;
    protected final static int DEFAULTSIZE = 5;
    private final static SpriteView view = new SpriteView();

    //some geometrical information
    float positionX = DEFAULTSIZE;
    float positionY = DEFAULTSIZE;
    float oldX, oldY;
    protected double speed = 0;
    protected float radius = 0;
    protected int direction = SOUTH; //in degrees from SOUTH
	 Color color = null;
    JPanel environment = null;
    
     Vector vector = new Vector();

    protected Sprite(JPanel panel) {
    	  environment = panel;
        addObserver(view);
    }

    /** Sets the direction of sprite */
    public void heading (int dir) {
        direction = dir;
    }

    /** Sets the speed between 1 and MAXSPEED */
    public void setSpeed (double sp) {
        if ((sp >= 1) && (sp <= MAXSPEED))
            speed = sp;
        else if (sp > MAXSPEED) speed = sp;
        //else if (sp < 0) speed = 0;
        //else if (sp <0) {direction = (direction + 180) % 360; speed = Math.abs(sp);}
    }
    // return x and y component of the vector
    public int vectorX(){return (int) (speed  * Math.cos( direction / 180.0 * Math.PI));}    
    public int vectorY(){return (int) (speed  * Math.sin( direction / 180.0 * Math.PI));}
   
    public void setdrawextra(){drawextra = 1;}
    public void cleardrawextra(){drawextra = 0;}
       public double sizeOfPoint(Point p) {
    return Math.sqrt(Math.pow(p.x, 2) + Math.pow(p.y, 2));
  }
       double getX(){return positionX;}
       double getY(){return positionY;}

    /** The tick method is called by our parent on every time tick.
        Here's where we it moves the sprite along its direction if necessary. */
    public void tick () {
        if (speed > 0){
            oldX = positionX;
            oldY = positionY;
            positionX += ((float)speed / MAXSPEED) * DELTA * Math.cos( direction / 180.0 * Math.PI);
            positionY += ((float)speed / MAXSPEED) * DELTA * Math.sin( direction / 180.0 * Math.PI);
        }

        

        // check for boundary bounce conditions
        if ((positionX - radius) <= 0) {
            bounceVertical();
            positionX = radius;
        }
        if ((positionX + radius) >= environment.getSize().width) {
            bounceVertical();
            positionX = environment.getSize().width - radius - 5;
        }
        if ((positionY - radius) <= 0) {
            bounceHorizontal();
            positionY = radius;
        }
        if ((positionY + radius) >= environment.getSize().height) {
            bounceHorizontal();
            positionY = environment.getSize().height - radius - 5;
        } 
    
         
    }
    
     public void reversetick () {                              
           positionX = oldX;
           positionY =  oldY;
        /*if (speed > 0){
            //positionX -= ((float)speed / MAXSPEED) * DELTA * Math.cos( direction / 180.0 * Math.PI);
            positionY -= ((float)speed / MAXSPEED) * DELTA * Math.sin( direction / 180.0 * Math.PI);
        }    
          
         */   
     }

    /** Causes the sprite to bounce off the vertical wall */
    public void bounceVertical() {
        direction = (180 - direction) % 360;
    }

    /** Causes the sprite to bounce off the horizontal wall */
    public void bounceHorizontal() {
        direction = (0 - direction) % 360;
    }

    public void paint(Graphics g) {
        
        double dx = ((float)speed / MAXSPEED) * DELTA * Math.cos( direction / 180.0 * Math.PI);
        double dy =   ((float)speed / MAXSPEED) * DELTA * Math.sin( direction / 180.0 * Math.PI);
        int vx = (int) (positionX +  dx * 20);
        int vy = (int) (positionY +  dy * 20) ;
        
        
        
      if (drawextra == 1)
      {
        g.setColor(Color.darkGray);
        g.drawLine((int) positionX, (int) positionY, vx, vy);
        g.setColor(color);
        
      int counter  = 0;
      for (Enumeration e = vector.elements() ; e.hasMoreElements() ;) {
          Point p = (Point)e.nextElement();
          
          counter++;
          switch(counter) {
              case 0: g.setColor(Color.ORANGE);
              break;
              case 1: g.setColor(Color.BLUE);
              break;
              case 2: g.setColor(Color.CYAN);
              break;
              case 3: g.setColor(Color.GREEN);
              break;
              case 4: g.setColor(Color.MAGENTA);
              break;
              case 5: g.setColor(Color.YELLOW);
              break;
              case 6: g.setColor(Color.YELLOW);
              break;
              case 7: g.setColor(Color.PINK);
              break;
              
              
          }
          
          g.drawLine((int)positionX,(int) positionY,(int)( positionX + p.getX()), (int)(positionY+p.getY()));
      }
      }

        g.setColor(Color.darkGray);
        g.fillArc((int)(positionX - 25), (int)(positionY - 25), 50, 50, (360 - direction ) ,5);
        
         setChanged();         
			notifyObservers(g);
                        
      
    }
}

/** a view class for sprite */
class SpriteView implements Observer {

   public void update(Observable o, Object arg) {
      Graphics g = (Graphics) arg;
      Sprite s = (Sprite) o;

      g.setColor(s.color);
      g.fillOval((int)(s.positionX - s.radius), (int)(s.positionY - s.radius), (int)(s.radius*2), (int)(s.radius*2));

      
   }
}
